#include <stdlib.h>
#include <stdio.h>

#include "util.h"
#include "sql.h"

/*
 * command parsing
 */

klist *klist_init(char *db) {
    klist *ctx = malloc(sizeof(klist));
    ctx->stmts = malloc(sizeof(sqlite3_stmt *) * _KLIST_SQL_COUNT);
    ctx->cmd_ctx = NULL;
    ctx->error = 0;

    klist_sql_prepare(ctx, db);

    return ctx;
}
void klist_deinit(klist *ctx) {
    sqlite3_close(ctx->db);
    if (ctx->cmd_ctx) free(ctx->cmd_ctx);
    free(ctx->stmts);
    free(ctx);
}

klist_user_context *klist_user_context_init(klist *list) {
    klist_user_context *ctx = malloc(sizeof(klist_user_context));
    ctx->cmd = -1;
    return ctx;
}
void klist_user_context_deinit(klist_user_context *ctx) {
    free(ctx);
}

klist_list_context *klist_list_context_init(klist *list) {
    klist_list_context *ctx = malloc(sizeof(klist_list_context));
    ctx->cmd = -1;
    ctx->name = NULL;
    ctx->stages = NULL;
    ctx->stages_len = 0;
    return ctx;
}
void klist_list_context_deinit(klist_list_context *ctx) {
    int i = 0;
    for (; i < ctx->stages_len; i++) free(ctx->stages[i]);
    free(ctx->stages);
    free(ctx);
}

klist_task_context *klist_task_context_init(klist *list) {
    klist_task_context *ctx = malloc(sizeof(klist_task_context));
    ctx->cmd = -1;
    ctx->name = NULL;
    ctx->stage = NULL;
    return ctx;
}
void klist_task_context_deinit(klist_task_context *ctx) {
    free(ctx->name);
    free(ctx->stage);
    free(ctx);
}

/*
 * sql preparations
 */

void klist_sql_prepare(klist *ctx, char *db) {
    ctx->stmts = malloc(sizeof(sqlite3_stmt *) * _KLIST_SQL_COUNT);
    sqlite3_open(db ? db : ":memory:", &ctx->db);

    int i = 0;
    for (; i < _KLIST_SQL_COUNT; i++)
        if (sqlite3_prepare(ctx->db, klist_sql_get(i), 0, &ctx->stmts[i], NULL) != SQLITE_OK)
            fprintf(stderr, "sqlite3_prepare: %s\n", sqlite3_errmsg(ctx->db));

}

/*
 * assuring things
 */

bool klist_assure_user(klist *, __uid_t, char *);
void klist_assure_list(klist *, char *);
void klist_assure_task(klist *, char *);